package ch.hslu.appe.inventory.persistency;

import java.util.ArrayList;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.Restrictions;

import ch.hslu.appe.inventory.model.Product;
import ch.hslu.appe.inventory.model.StockOrder;
import ch.hslu.appe.persistency.Persistency;

/**
 * Implements persistency for the Inventory / Stock.
 * @author Felix Rohrer <felix.rohrer@stud.hslu.ch>
 */
public class InventoryPersistencyImpl extends Persistency implements InventoryPersistency {

    /**
     * Constructor.
     */
    public InventoryPersistencyImpl() {
        super("InventoryData");
    }

    @Override
    public final boolean addProduct(final int newProduct) {
        // TODO Auto-generated method stub
        logger.trace("addProduct() is not yet implemented!");
        return false;
    }

    @Override
    public final boolean addStockOrder(final StockOrder stockOrder) {

        final Session session = sessionFactory.openSession();
        Transaction transaction = null;

        try {
            // Begin a transaction
            transaction = getTransaction(session);

            stockOrder.setId((Integer) session.save(stockOrder));

            // System.out.println("# New StockOrderId: '" + stockOrder.getId() + "'");

            // Commit the transaction
            if (globalTransaction == null) {
                transaction.commit();
            }

            // Add log trace entry
            logger.trace("Persisted a new StockOrder '" + stockOrder + "' successfully");

            // Flush, to make sure, that the object is saved
            session.flush();

        } catch (HibernateException e) {
            // Roll back the transaction
            if (transaction != null && globalTransaction == null) {
                transaction.rollback();
                logger.trace("Roll back transaction due an internal error");
            }

            // Log the error
            logger.error("Failed to save the StockOrder '" + stockOrder + "'.", e);

            return false;
        } finally {
            session.close();
        }
        return true;

    }

    @Override
    public final Product getProduct(final int productNumber) {
        // Ignore empty invalid id's
        if (productNumber <= 0) {
            return null;
        }

        final Session session = sessionFactory.openSession();
        try {

            final Criteria criteria = session.createCriteria(Product.class);

            // Add the criteria (OrderNumber must match)
            criteria.add(Restrictions.eq("number", productNumber));

            // Get the order by orderNumber
            final Product product = (Product) criteria.uniqueResult();

            // Add log trace entry
            logger.trace("Fetched the product '" + product + "' sucessfully");

            return product;
        } finally {
            session.close();
        }
    }

    @Override
    public final ArrayList<Product> getProducts() {
        final Session session = sessionFactory.openSession();
        final ArrayList<Product> products = new ArrayList<Product>();

        try {
            final Criteria criteria = session.createCriteria(Product.class);

            // Add the filter
            // no filter implemented so far

            // Fetch all db users
            @SuppressWarnings("rawtypes")
            final List fetchedProducts = criteria.list();

            // Add order to the typed generic list
            for (Object product : fetchedProducts) {
                products.add((Product) product);
            }

            // Add log trace entry
            logger.trace("Fetched " + products.size() + " products sucessfully");
        } finally {
            session.close();
        }
        return products;
    }

    @Override
    public final List<Product> getProductsActive() {
        final Session session = sessionFactory.openSession();
        final ArrayList<Product> products = new ArrayList<Product>();

        try {
            final Criteria criteria = session.createCriteria(Product.class);

            // Fetch all db users
            @SuppressWarnings("rawtypes")
            final List fetchedProducts = criteria.list();

            // Add order to the typed generic list
            for (Object product : fetchedProducts) {
                if (((Product) product).isActive()) {

                    products.add((Product) product);
                }
            }

            // Add log trace entry
            logger.trace("Fetched " + products.size() + " active products sucessfully");
        } finally {
            session.close();
        }
        return products;
    }

    @Override
    public final StockOrder getStockOrder(final int stockOrderId) {
        // Ignore empty invalid id's
        if (stockOrderId <= 0) {
            return null;
        }

        final Session session = sessionFactory.openSession();
        try {

            final Criteria criteria = session.createCriteria(StockOrder.class);

            // Add the criteria (StockOrderId must match)
            criteria.add(Restrictions.eq("id", stockOrderId));

            // Get the order by orderNumber
            final StockOrder stockOrder = (StockOrder) criteria.uniqueResult();

            // Add log trace entry
            logger.trace("Fetched the StockOrder '" + stockOrder + "' sucessfully");

            return stockOrder;
        } finally {
            session.close();
        }
    }

    @Override
    public final boolean productActive(final int productNumber) {
        // Ignore empty invalid id's
        if (productNumber <= 0) {
            return false;
        }
        // fetch product
        final Product product = getProduct(productNumber);
        if (product == null) {
            return false;
        }

        final boolean productState = product.isActive();

        // Add log trace entry
        logger.trace("Fetched the product '" + product + "', state '" + productState + "' sucessfully");

        // return state
        return productState;
    }

    @Override
    public final boolean productExists(final int productNumber) {
        // Ignore empty invalid id's
        if (productNumber <= 0) {
            return false;
        }
        // fetch product
        final Product product = getProduct(productNumber);
        if (product == null) {
            return false;
        }

        // Add log trace entry
        logger.trace("Fetched the product '" + product + "' sucessfully -> Product exists!");

        return true;
    }

    @Override
    public final boolean setProductActiveState(final int productNumber, final boolean newState) {
        // Ignore empty invalid id's
        if (productNumber <= 0) {
            return false;
        }
        // fetch product
        final Product product = getProduct(productNumber);
        if (product == null) {
            return false;
        }

        // update Active State
        product.setActive(newState);

        // Add log trace entry
        logger.trace("Updated the product '" + product + "' state to '" + newState + "' sucessfully");

        // save product update
        return updateProduct(product);
    }

    @Override
    public final boolean updateProduct(final Product newProduct) {
        final Product product = getProduct(newProduct.getNumber());
        if (product == null) {
            logger.trace("The Product with the ProductNumber '" + newProduct.getNumber()
                    + "' was not found. NO Update executed!");
            return false;
        }

        // Update Product info
        product.setActive(newProduct.isActive());
        product.setAmount(newProduct.getAmount());
        product.setAmountmin(newProduct.getAmountmin());
        product.setPrice(newProduct.getPrice());
        product.setProductname(newProduct.getProductname());

        // save Product info
        final Session session = sessionFactory.openSession();
        Transaction transaction = null;
        try {
            // Begin a transaction
            transaction = getTransaction(session);

            session.update(product);

            // Commit the transaction
            if (globalTransaction == null) {
                transaction.commit();
            }

            // Flush, to make sure, that the object is saved
            session.flush();

            // Add log trace entry
            logger.trace("Updated the product '" + product + "' sucessfully");

        } catch (HibernateException e) {
            // Roll back the transaction
            if (transaction != null && globalTransaction == null) {
                transaction.rollback();
                logger.trace("Roll back transaction due an internal error");
            }

            // Log the error
            logger.error("Failed to update the product '" + product + "'.", e);
            return false;
        } finally {
            session.close();
        }
        return true;
    }

    @Override
    public final boolean updateStock(final int productNumber, final int newStock) {
        // Ignore empty invalid id's
        if (productNumber <= 0) {
            return false;
        }
        // fetch product
        final Product product = getProduct(productNumber);
        if (product == null) {
            return false;
        }

        // update Stock Amount
        product.setAmount(newStock);

        // Add log trace entry
        logger.trace("Updated the product '" + product + "' stock amount to '" + newStock + "' sucessfully");

        // save product update
        return updateProduct(product);
    }

    @Override
    public final boolean updateStockOrder(final StockOrder stockOrder) {
        // TODO Auto-generated method stub
        logger.trace("updateStockOrder() is not yet implemented!");
        return false;
    }

}
