package ch.hslu.appe.persistency;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.boot.registry.StandardServiceRegistryBuilder;
import org.hibernate.cfg.Configuration;
import org.hibernate.service.ServiceRegistry;

/**
 * Persistency Helper Class.
 */
public abstract class Persistency {

    protected static SessionFactory sessionFactory;
    protected static ServiceRegistry serviceRegistry;
    protected static Logger logger;
    protected Transaction globalTransaction;

    /**
     * Constructor.
     * @param loggerName logger name
     */
    protected Persistency(final String loggerName) {

        logger = LogManager.getLogger(loggerName);

        if (sessionFactory == null) {
            try {
                // Configure the session
                final Configuration configuration = new Configuration();
                configuration.configure();

                serviceRegistry = new StandardServiceRegistryBuilder().applySettings(configuration.getProperties())
                        .build();
                sessionFactory = configuration.buildSessionFactory(serviceRegistry);
            } catch (Throwable ex) {
                logger.error("Failed to create sessionFactory object", ex);

                throw new ExceptionInInitializerError(ex);
            }
        }
    }

    /**
     * Commits the global transaction.
     * @return Returns true on success, otherwise false
     */
    public final boolean commitGlobalTransaction() {

        if (globalTransaction == null) {

            logger.error("There is not global transaction to commit!");
            return false;
        }

        try {

            // Start the transaction
            globalTransaction.commit();

        } catch (HibernateException ex) {

            logger.error("Cannot commit a global transaction!", ex);

            return false;
        } finally {
            globalTransaction = null;
        }

        return true;
    }

    /**
     * Gets a transaction. If there's a global transaction atm, it will take this, otherwise it will generate a new
     * transaction
     * @param session SessionObject
     * @return Transaction
     */
    protected final Transaction getTransaction(final Session session) {

        if (globalTransaction != null) {
            return globalTransaction;
        }

        return session.beginTransaction();
    }

    /**
     * Rolls the current global transaction back.
     * @return Returns true on success, otherwise false
     */
    public final boolean rollbackGlobalTransaction() {

        if (globalTransaction == null) {

            logger.error("There is not global transaction to roll back!");
            return false;
        }

        try {

            // Start the transaction
            globalTransaction.rollback();

        } catch (HibernateException ex) {

            logger.error("Cannot roll back a global transaction!", ex);

            return false;
        } finally {
            globalTransaction = null;
        }

        return true;
    }

    /**
     * Starts a global transaction.
     * @return Returns true on success, otherwise false
     */
    public final boolean startGlobalTransaction() {

        if (globalTransaction != null) {

            logger.error("Cannot start a new global transaction unless the previous has been commited or roll backed!");
            return false;
        }

        final Session session = sessionFactory.openSession();

        try {

            // Start the transaction
            globalTransaction = session.beginTransaction();

        } catch (HibernateException ex) {

            logger.error("Cannot start a global transaction!", ex);

            return false;
        } finally {

            session.close();
        }

        return true;
    }
}
