package ch.hslu.appe.usermanagement.model;

import java.util.List;

import ch.hslu.appe.usermanagement.exception.IllegalNameException;

/**
 * User (Customer / Employee).
 */
public final class User {

    private int id;
    private String login;
    private String password;
    private String prename;
    private String role;
    private String salutation;
    private UserState state;
    private String surname;
    private List<Address> addresses;

    @Override
    public boolean equals(final Object other) {

        // Check for same instance
        if (this == other) {
            return true;
        }

        // Check for null values
        if (other == null) {
            return false;
        }

        // Check for same type
        if (other.getClass() != getClass()) {
            return false;
        }

        // Check properties
        return ((User) other).getId() == getId() && ((User) other).getLogin().equals(getLogin());
    }

    /**
     * returns all addresses.
     * @return list with all addresses
     */
    public List<Address> getAddresses() {
        return addresses;
    }

    /**
     * Gets the unique user id, generated by the database.
     * @return Returns the id
     */
    public int getId() {
        return id;
    }

    /**
     * Gets the login name.
     * @return Returns the login name
     */
    public String getLogin() {
        return login;
    }

    /**
     * Gets the password (Salted MD5 Hash).
     * @return Returns the hashed password
     */
    public String getPassword() {
        return password;
    }

    /**
     * Gets the prename.
     * @return Returns the prename
     */
    public String getPrename() {
        return prename;
    }

    /**
     * Gets the role (admin, customer, etc.).
     * @return Returns the administrative role
     */
    public String getRole() {
        return role;
    }

    /**
     * Gets the salutation.
     * @return Returns the salutation
     */
    public String getSalutation() {
        return salutation;
    }

    /**
     * Gets the user state.
     * @return Returns the user state
     */
    public UserState getState() {
        return state;
    }

    /**
     * Gets the surname.
     * @return Returns the surname
     */
    public String getSurname() {
        return surname;
    }

    @Override
    public int hashCode() {
        return id + login.hashCode();
    }

    /**
     * Add a new Address.
     * @param addresses address to be added
     */
    public void setAddresses(final List<Address> addresses) {
        this.addresses = addresses;
    }

    /**
     * Sets the user id.
     * @param id The user id
     */
    public void setId(final int id) {
        this.id = id;
    }

    /**
     * Sets the user login.
     * @param login The login name
     * @throws IllegalNameException illegal username
     */
    public void setLogin(final String login) throws IllegalNameException {

        if (login == null || login.matches("[\\w.]+")) {
            this.login = login;
        } else {

            // Throw illegal login exception
            throw new IllegalNameException(login);
        }
    }

    /**
     * Sets the password.
     * @param password The password
     */
    public void setPassword(final String password) {
        this.password = password;
    }

    /**
     * Sets the prename.
     * @param prename The prename
     */
    public void setPrename(final String prename) {
        this.prename = prename;
    }

    /**
     * Sets the role.
     * @param role The role
     */
    public void setRole(final String role) {
        this.role = role;
    }

    /**
     * Sets the salutation (mr., mrs., dr., etc.).
     * @param salutation The salutation
     */
    public void setSalutation(final String salutation) {
        this.salutation = salutation;
    }

    /**
     * Sets the user state.
     * @param state The user state
     */
    public void setState(final UserState state) {
        this.state = state;
    }

    /**
     * Sets the surname.
     * @param surname The surname
     */
    public void setSurname(final String surname) {
        this.surname = surname;
    }

    @Override
    public String toString() {
        return login + "(" + id + ", " + surname + ", " + prename + ")";
    }
}
