package ch.hslu.appe.usermanagement.persistency;

import java.util.ArrayList;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.Restrictions;

import ch.hslu.appe.persistency.Persistency;
import ch.hslu.appe.usermanagement.exception.MultipleHomeAddressesException;
import ch.hslu.appe.usermanagement.model.Address;
import ch.hslu.appe.usermanagement.model.AddressType;
import ch.hslu.appe.usermanagement.model.User;

/**
 * Address Persistency Implementation.
 */
public final class AddressPersistencyImpl extends Persistency implements AddressPersistency {

    private static final UserPersistencyImpl USERPERSISTENCY = new UserPersistencyImpl();

    /**
     * Constructor.
     */
    public AddressPersistencyImpl() {

        super("AccountingData");
    }

    @Override
    public boolean addAddress(final int userId, final Address address) throws MultipleHomeAddressesException {

        final Session session = sessionFactory.openSession();
        Transaction transaction = null;

        if (!USERPERSISTENCY.userExists(userId)) {

            logger.error("Addresse konnte nicht hinzugefügt werden, da kein Benutzer mit der Id '" + userId
                    + "' existiert.");
            return false;
        }

        try {

            // User can have only one home address
            if (address.getType() == AddressType.HOME) {

                // Count the user home addresses, if there's already another
                // one, raise an exception
                final Long count = (Long) session.createQuery(
                        "select count(*) from " + Address.class.getName() + " where type = "
                                + AddressType.HOME.ordinal() + " AND " + "userid = " + userId).uniqueResult();

                if (count.intValue() > 0) {
                    throw new MultipleHomeAddressesException();
                }
            }

            // Begin a transaction
            transaction = getTransaction(session);

            final User addressUser = new User();
            addressUser.setId(userId);
            address.setUser(addressUser);

            address.setId((Integer) session.save(address));

            // Commit the transaction
            if (globalTransaction == null) {
                transaction.commit();
            }

            // Add log trace entry
            logger.trace("Persisted a new address '" + address + "' successfully");

            // Flush, to make sure, that the object is saved
            session.flush();

            return true;
        } catch (Exception ex) {

            // Roll back the transaction
            if (transaction != null && globalTransaction == null) {
                transaction.rollback();

                logger.trace("Roll back transaction due an internal error");
            }

            // Log the error
            logger.error("Failed to save the address '" + address + "'.", ex);

            return false;
        } finally {
            session.close();
        }
    }

    @Override
    public boolean addressExists(final Address address) {

        // Check for null values
        if (address == null) {
            return false;
        }

        // Check validity of the address id
        if (address.getId() <= 0) {
            return false;
        }

        final Session session = sessionFactory.openSession();

        try {
            final Query query = session.createQuery("select 1 from Address t where t.id = :id");

            query.setInteger("id", address.getId());

            // If there's at least one result, return true, otherwise false
            final boolean exists = (query.uniqueResult() != null);

            // Add log trace entry
            logger.trace("Check for address '" + address + "' existency: " + exists);

            return exists;
        } finally {
            session.close();
        }
    }

    @Override
    public boolean deleteAddress(final int addressId) {
        final Session session = sessionFactory.openSession();
        Transaction transaction = null;

        final Address address = getAddress(addressId);

        // Can only delete existent addresses
        if (address == null) {
            return false;
        }

        // Check constraint: Can't delete users home address
        if (address.getType() == AddressType.HOME) {
            return false;
        }

        try {

            // Begin a transaction
            transaction = getTransaction(session);

            // Create a hql query and execute it against the db
            final String hql = "delete from " + User.class.getName() + " where id= :id";
            session.createQuery(hql).setInteger("id", addressId).executeUpdate();

            // Commit the transaction
            if (globalTransaction == null) {
                transaction.commit();
            }

            // Flush, to make sure, that the object is saved
            session.flush();

            // Add log trace entry
            logger.trace("Deleted the address with the id '" + addressId + "' sucessfully");
        } catch (HibernateException e) {

            // Roll back the transaction
            if (transaction != null && globalTransaction == null) {
                transaction.rollback();

                logger.trace("Roll back transaction due an internal error");
            }

            // Log the error
            logger.error("Failed to update the address with the id '" + addressId + "'.", e);

            return false;
        } finally {
            session.close();
        }

        return true;
    }

    @Override
    public Address getAddress(final int addressId) {

        // Ignore empty invalid id's
        if (addressId <= 0) {
            return null;
        }

        final Session session = sessionFactory.openSession();

        try {
            final Criteria criteria = session.createCriteria(Address.class);

            // Add the criteria (Id must match)
            criteria.add(Restrictions.eq("id", addressId));

            final Address address = (Address) criteria.uniqueResult();

            // Add log trace entry
            logger.trace("Fetched the address '" + address + "' sucessfully");

            return address;
        } finally {

            session.close();
        }
    }

    @Override
    public ArrayList<Address> getUserAddresses(final int userId) {

        // Ignore empty invalid id's
        if (userId <= 0) {
            return null;
        }

        final ArrayList<Address> addresses = new ArrayList<Address>();

        final Session session = sessionFactory.openSession();

        try {
            final Criteria criteria = session.createCriteria(Address.class);

            // Add the criteria (The foreign key user id must match)
            criteria.add(Restrictions.eq("user.id", userId));

            // Fetch all db addresses
            @SuppressWarnings("rawtypes")
            final List fetchedAddresses = criteria.list();

            // Add users to the typed generic list
            for (Object address : fetchedAddresses) {
                addresses.add((Address) address);
            }

            // Add log trace entry
            logger.trace("Fetched " + addresses.size() + " addresses sucessfully");
        } finally {

            session.close();
        }

        return addresses;
    }

    @Override
    public boolean updateAddress(final Address address) {

        final Session session = sessionFactory.openSession();
        Transaction transaction = null;

        try {
            // Begin a transaction
            transaction = getTransaction(session);

            session.update(address);

            // Commit the transaction
            if (globalTransaction == null) {
                transaction.commit();
            }

            // Flush, to make sure, that the object is saved
            session.flush();

            // Add log trace entry
            logger.trace("Updated the address '" + address + "' sucessfully");
        } catch (HibernateException e) {

            // Roll back the transaction
            if (transaction != null && globalTransaction == null) {
                transaction.rollback();

                logger.trace("Roll back transaction due an internal error");
            }

            // Log the error
            logger.error("Failed to update the address '" + address + "'.", e);

            return false;
        } finally {
            session.close();
        }

        return true;
    }

}
