package ch.hslu.appe.usermanagement.persistency;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runners.model.InitializationError;

import ch.hslu.appe.usermanagement.exception.IllegalNameException;
import ch.hslu.appe.usermanagement.exception.MultipleHomeAddressesException;
import ch.hslu.appe.usermanagement.model.Address;
import ch.hslu.appe.usermanagement.model.AddressType;
import ch.hslu.appe.usermanagement.model.Place;
import ch.hslu.appe.usermanagement.model.User;
import ch.hslu.appe.usermanagement.model.UserState;
import ch.hslu.appe.usermanagement.persistency.AddressPersistency;
import ch.hslu.appe.usermanagement.persistency.AddressPersistencyImpl;
import ch.hslu.appe.usermanagement.persistency.UserPersistency;
import ch.hslu.appe.usermanagement.persistency.UserPersistencyImpl;

/**
 * Address Persistency Implementation Test.
 */
public class AddressPersistencyImplTest {

    private User user;
    private Place place;
    private Address homeAddress;
    private final UserPersistency userPersistency = new UserPersistencyImpl();
    private final AddressPersistency addressPersistency = new AddressPersistencyImpl();

    /**
     * Create Business Address Helperfunction.
     * @return BusinessAddress
     */
    private Address createBusinessAddress() {
        final Address address = new Address();

        address.setPlace(place);
        address.setStreet("Bebenstrasse 487");
        address.setType(AddressType.BUSINESS);

        return address;
    }

    /**
     * Add HomeAddress HelperFunction.
     * @throws MultipleHomeAddressesException User has already a home address
     */
    private void insertHomeAddress() throws MultipleHomeAddressesException {
        if (homeAddress == null) {
            homeAddress = new Address();

            homeAddress.setPlace(place);
            homeAddress.setStreet("Bebenstrasse 487");
            homeAddress.setType(AddressType.HOME);

            Assert.assertTrue(addressPersistency.addAddress(user.getId(), homeAddress));
        }
    }

    /**
     * JUnit Setup.
     * @throws IllegalNameException illegal Username
     * @throws InitializationError Init Error
     */
    @Before
    public final void setup() throws IllegalNameException, InitializationError {

        place = new Place();
        place.setId(104);

        // Create a test user
        user = new User();

        user.setLogin("TestUser" + System.currentTimeMillis());
        user.setPrename("Hans");
        user.setSalutation("Herr");
        user.setSurname("Imboden");
        user.setRole("admin");
        user.setState(UserState.ACTIVE);

        if (!userPersistency.addUser(user)) {
            throw new InitializationError("Couldn't setup the preconditions of the unit tests!");
        }
    }

    /**
     * JUnit TearDown.
     */
    @After
    public final void tearDown() {

        if (user.getId() > 0) {
            userPersistency.deleteUser(user.getId());
        }
    }

    /**
     * Test Add Address.
     * @throws MultipleHomeAddressesException user has multiple home addresses
     */
    @Test
    public final void testAddAddress() throws MultipleHomeAddressesException {

        final Address address = createBusinessAddress();

        Assert.assertTrue(addressPersistency.addAddress(user.getId(), address));
    }

    /**
     * Test that no Address exists.
     */
    @Test
    public final void testAddressExistsFalse() {

        Assert.assertFalse(addressPersistency.addressExists(new Address()));
    }

    /**
     * Tests that a address exists.
     * @throws MultipleHomeAddressesException user has multiple home addresses
     */
    @Test
    public final void testAddressExistsTrue() throws MultipleHomeAddressesException {

        final Address address = createBusinessAddress();

        Assert.assertTrue(addressPersistency.addAddress(user.getId(), address));
        Assert.assertTrue(addressPersistency.addressExists(address));
    }

    /**
     * Tests to add a address to a non existent user.
     * @throws MultipleHomeAddressesException user has multiple home addresses
     */
    @Test
    public final void testAddressUserNotExists() throws MultipleHomeAddressesException {

        final Address address = createBusinessAddress();

        // Expected to receive false, when trying add an address to a non
        // existent user
        Assert.assertFalse(addressPersistency.addAddress(-1, address));
    }

    /**
     * Test to add a second Home Address.
     * @throws MultipleHomeAddressesException user has multiple home addresses
     */
    @Test
    public final void testAddSecondHomeAddress() throws MultipleHomeAddressesException {

        insertHomeAddress();

        final Address secondHomeAddress = createBusinessAddress();
        secondHomeAddress.setType(AddressType.HOME);

        // Try to add the second home address
        Assert.assertFalse(addressPersistency.addAddress(user.getId(), secondHomeAddress));
    }

    /**
     * Test to delete a address.
     * @throws MultipleHomeAddressesException user has multiple home addresses
     */
    @Test
    public final void testDeleteAddress() throws MultipleHomeAddressesException {
        final Address address = createBusinessAddress();

        // Add and delete the address
        Assert.assertTrue(addressPersistency.addAddress(user.getId(), address));
        Assert.assertTrue(addressPersistency.deleteAddress(address.getId()));
    }

    /**
     * Test to delete a Home Address.
     * @throws MultipleHomeAddressesException user has multiple home addresses
     */
    @Test
    public final void testDeleteHomeAddress() throws MultipleHomeAddressesException {

        insertHomeAddress();

        // It should not be possible to delete the home address
        Assert.assertFalse(addressPersistency.deleteAddress(homeAddress.getId()));
    }

    /**
     * Test get Addresses.
     * @throws MultipleHomeAddressesException user has multiple home addresses
     */
    @Test
    public final void testGetAddresses() throws MultipleHomeAddressesException {

        insertHomeAddress();

        // Expected to get one or more user addresses
        Assert.assertTrue(addressPersistency.getUserAddresses(user.getId()).size() > 0);
    }

    /**
     * Test get non existent User Addresses.
     */
    @Test
    public final void testGetNonExistentUserAddresses() {

        // Expected to get an empty result set when looking up addresses of a
        // non existent user
        Assert.assertEquals(null, addressPersistency.getUserAddresses(-1));
    }

    /**
     * Test update Address.
     * @throws MultipleHomeAddressesException user has multiple home addresses
     */
    @Test
    public final void updateAddress() throws MultipleHomeAddressesException {

        final Address address = createBusinessAddress();

        Assert.assertTrue(addressPersistency.addAddress(user.getId(), address));

        // Update the address
        address.setStreet("Belchenweg 34");
        Assert.assertTrue(addressPersistency.updateAddress(address));

        final Address insertedAddress = addressPersistency.getAddress(address.getId());
        Assert.assertEquals("Belchenweg 34", insertedAddress.getStreet());
    }

    /**
     * Test Update Home Address.
     * @throws MultipleHomeAddressesException user has multiple home addresses
     */
    @Test
    public final void updateHomeAddress() throws MultipleHomeAddressesException {

        insertHomeAddress();

        homeAddress.setStreet("Lerchenweg 34");

        Assert.assertTrue(addressPersistency.updateAddress(homeAddress));
    }
}
